Package: PathwaySpace
Type: Package
Title: Spatial Projection of Network Signals along Geodesic Paths
Version: 1.3.0
Authors@R: 
    c(person("Sysbiolab Team", role = "aut"),
      person("Victor", "Apolonio", role = "ctb"),
      person("Jonathan", "Back", role = "ctb"),
      person("Lana", "Querne", role = "ctb"),
      person("Vinicius", "Chagas", role = "ctb"),
      person("Bahar", "Tercan", role = "ctb"),
      person("Mauro", "Castro", role = c("cre"), 
              email = "mauro.a.castro@gmail.com", 
              comment = c(ORCID = "0000-0003-4942-8131"))
      )
Depends: R(>= 4.5), methods, RGraphSpace(>= 1.2)
Imports: grDevices, stats, scales, RANN, igraph, ggplot2, ggrepel,
        grid, colorspace, patchwork, lifecycle
Suggests: knitr, rmarkdown, testthat, RedeR
Description: For a given graph containing vertices, edges, and a signal 
    associated with the vertices, the 'PathwaySpace' package performs a convolution 
    operation, which involves a weighted combination of neighboring vertices and 
    their associated signals. The package uses a decay function to project these 
    signals, creating geodesic paths on a 2D-image space. 'PathwaySpace' has various 
    applications, such as visualizing network data in a graphical format that 
    highlights the relationships and signal strengths between vertices. By 
    combining graph theory, signal processing, and visualization, 'PathwaySpace' 
    provides a way of representing graph data on a continuous projection space.
    Based on methods introduced in Tercan et al. (2025) 
    <doi:10.1016/j.xpro.2025.103681> and Ellrott et al. (2025) 
    <doi:10.1016/j.ccell.2024.12.002>.
License: Artistic-2.0
VignetteBuilder: knitr
URL: https://sysbiolab.github.io/PathwaySpace/,
        https://github.com/sysbiolab/PathwaySpace
BugReports: https://github.com/sysbiolab/PathwaySpace/issues
Collate: pspaceChecks.R pspaceSupplements.R pspaceMisc.R
        pspaceWatershed.R pspaceDecay.R pspaceClasses.R
        pspaceGenerics.R pspaceMethods.R pspacePlots.R
Encoding: UTF-8
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-23 14:40:01 UTC; maac
Author: Sysbiolab Team [aut],
  Victor Apolonio [ctb],
  Jonathan Back [ctb],
  Lana Querne [ctb],
  Vinicius Chagas [ctb],
  Bahar Tercan [ctb],
  Mauro Castro [cre] (ORCID: <https://orcid.org/0000-0003-4942-8131>)
Maintainer: Mauro Castro <mauro.a.castro@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-23 15:20:02 UTC
Built: R 4.5.2; ; 2026-05-25 02:33:35 UTC; unix
