# Generated by roxygen2: do not edit by hand

export(buildPathwaySpace)
export(expDecay)
export(getNearestNode)
export(linearDecay)
export(pathDistances)
export(plotPathDistances)
export(polarDecay)
export(pspace.cols)
export(pspace.pals)
export(signalAggregation)
export(signalDecay)
export(summitWatershed)
export(weibullDecay)
exportClasses(PathwaySpace)
exportMethods("gs_edge_attr<-")
exportMethods("gs_vertex_attr<-")
exportMethods("vertexDecay<-")
exportMethods("vertexSignal<-")
exportMethods(circularProjection)
exportMethods(getPathwaySpace)
exportMethods(plotPathwaySpace)
exportMethods(polarProjection)
exportMethods(silhouetteMapping)
exportMethods(summitMapping)
exportMethods(vertexDecay)
exportMethods(vertexSignal)
import(methods)
importClassesFrom(RGraphSpace,GraphSpace)
importFrom(RANN,nn2)
importFrom(RGraphSpace,"gs_edge_attr<-")
importFrom(RGraphSpace,"gs_vertex_attr<-")
importFrom(RGraphSpace,GraphSpace)
importFrom(RGraphSpace,getGraphSpace)
importFrom(RGraphSpace,gs_ecount)
importFrom(RGraphSpace,gs_edge_attr)
importFrom(RGraphSpace,gs_vcount)
importFrom(RGraphSpace,gs_vertex_attr)
importFrom(RGraphSpace,normalizeGraphSpace)
importFrom(RGraphSpace,plotGraphSpace)
importFrom(RGraphSpace,theme_gspace_coords)
importFrom(colorspace,lighten)
importFrom(ggplot2,aes)
importFrom(ggplot2,annotate)
importFrom(ggplot2,annotation_raster)
importFrom(ggplot2,coord_fixed)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,element_line)
importFrom(ggplot2,element_rect)
importFrom(ggplot2,element_text)
importFrom(ggplot2,expansion)
importFrom(ggplot2,geom_area)
importFrom(ggplot2,geom_hline)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_raster)
importFrom(ggplot2,geom_segment)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,labs)
importFrom(ggplot2,margin)
importFrom(ggplot2,rel)
importFrom(ggplot2,scale_fill_gradientn)
importFrom(ggplot2,scale_x_continuous)
importFrom(ggplot2,scale_y_continuous)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_bw)
importFrom(ggplot2,theme_gray)
importFrom(ggplot2,theme_minimal)
importFrom(ggplot2,xlab)
importFrom(ggplot2,ylab)
importFrom(ggrepel,geom_text_repel)
importFrom(grDevices,adjustcolor)
importFrom(grDevices,as.raster)
importFrom(grDevices,col2rgb)
importFrom(grDevices,colorRampPalette)
importFrom(grDevices,convertColor)
importFrom(grDevices,rgb)
importFrom(grid,unit)
importFrom(igraph,'E<-')
importFrom(igraph,'V<-')
importFrom(igraph,E)
importFrom(igraph,V)
importFrom(igraph,as_adjacency_matrix)
importFrom(igraph,as_edgelist)
importFrom(igraph,degree)
importFrom(igraph,distances)
importFrom(igraph,ecount)
importFrom(igraph,is_directed)
importFrom(igraph,is_igraph)
importFrom(igraph,simplify)
importFrom(igraph,vcount)
importFrom(igraph,which_mutual)
importFrom(lifecycle,deprecate_soft)
importFrom(lifecycle,deprecate_stop)
importFrom(lifecycle,deprecated)
importFrom(lifecycle,is_present)
importFrom(patchwork,wrap_plots)
importFrom(scales,rescale)
importFrom(stats,density)
importFrom(stats,median)
importFrom(stats,quantile)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,weighted.mean)
importFrom(utils,setTxtProgressBar)
importFrom(utils,txtProgressBar)
